//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Modul:       - Dieses Modul beinhaltet smtliche USB-Funktionen
//              - bietet Fkt zur Initialiserung und Konfiguration der Endpoints 
//                und PLL (auf 48MHz)
//              - behandelt die gesamte USB-Enumeration (Austausch der 
//                Descriptoren bei der Treiber-Initialisierung)
//              - behandelt alle USB-Interrupts, wickelt die Bulk-Transfers ab
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#ifndef _USB_H_
#define _USB_H_


//--- Konstanten fr die String-Deskriptoren
#define USB_MANUFACTURER_NAME {'A'<<8, 'N'<<8, 'D'<<8, 'I'<<8 }
#define USB_MN_LENGTH         4
#define USB_PRODUCT_NAME      {'A'<<8, 'n'<<8, 'd'<<8, 'i'<<8, 's'<<8, ' '<<8,  \
                               'U'<<8, 'S'<<8, 'B'<<8, '-'<<8, \
                               'P'<<8, 'r'<<8, 'o'<<8, 'g'<<8, 'r'<<8, 'a'<<8, 'm'<<8, 'm'<<8, 'e'<<8, 'r'<<8, ' '<<8 }
#define USB_PN_LENGTH         21
#define USB_SERIAL_NUMBER     {'1'<<8, '.'<<8, '0'<<8, '.'<<8, '0'<<8}
#define USB_SN_LENGTH         5

// Sprach-ID: 0x0904 = USA(default), 0x0407 = deutsch
#define LANGUAGE_ID           0x0407         
  

//--- USB REQUEST SETUP PACKET
typedef struct st_usb_request
{
  Uchar bmRequestType;                            // Characteristics of the request
  Uchar bRequest;                                 // Specific request
  Uchar wValueL;                                  // field that varies according to request
  Uchar wValueH;                                  
  Uchar wIndexL;                                  // field that varies according to request
  Uchar wIndexH;                                  
  Uchar wLengthL;                                 // Number of bytes to transfer if Data
  Uchar wLengthH;                                 
} t_usb_request;



//--- USB DEVICE DESCRIPTOR
typedef struct st_usb_device_descriptor
{
  Uchar  bLength;                                 // Size of this descriptor in bytes
  Uchar  bDescriptorType;                         // DEVICE descriptor type
  Uint16 bscUSB;                                  // Binay Coded Decimal Spec. release
  Uchar  bDeviceClass;                            // Class code assigned by the USB
  Uchar  bDeviceSubClass;                         // Sub-class code assigned by the USB
  Uchar  bDeviceProtocol;                         // Protocol code assigned by the USB
  Uchar  bMaxPacketSize0;                         // Max packet size for EP0
  Uint16 idVendor;                                // Vendor ID. ATMEL = 0x03EB
  Uint16 idProduct;                               // Product ID assigned by the manufacturer
  Uint16 bcdDevice;                               // Device release number
  Uchar  iManufacturer;                           // Index of manu. string descriptor
  Uchar  iProduct;                                // Index of prod. string descriptor
  Uchar  iSerialNumber;                           // Index of S.N.  string descriptor
  Uchar  bNumConfigurations;                      // Number of possible configurations
} t_usb_device_descriptor;



//--- USB CONFIGURATION DESCRIPTOR
typedef struct st_usb_configuration_descriptor
{
  Uchar  bLength;                                 // size of this descriptor in bytes
  Uchar  bDescriptorType;                         // CONFIGURATION descriptor type
  Uint16 wTotalLength;                            // total length of data returned
  Uchar  bNumInterfaces;                          // number of interfaces for this conf.
  Uchar  bConfigurationValue;                     // value for SetConfiguration resquest
  Uchar  iConfiguration;                          // index of string descriptor
  Uchar  bmAttibutes;                             // Configuration characteristics
  Uchar  MaxPower;                                // maximum power consumption
} t_usb_configuration_descriptor;
  
  
//--- USB INTERFACE DESCRIPTOR
typedef struct st_usb_interface_descriptor
{
  Uchar bLength;                                  // size of this descriptor in bytes
  Uchar bDescriptorType;                          // INTERFACE descriptor type
  Uchar bInterfaceNumber;                         // Number of interface
  Uchar bAlternateSetting;                        // value to select alternate setting
  Uchar bNumEndpoints;                            // Number of EP except EP 0
  Uchar bInterfaceClass;                          // Class code assigned by the USB
  Uchar bInterfaceSubClass;                       // Sub-class code assigned by the USB
  Uchar bInterfaceProtocol;                       // Protocol code assigned by the USB
  Uchar iInterface;                               // Index of string descriptor
} t_usb_interface_descriptor;   


//--- USB ENDPOINT DESCRIPTOR
typedef struct st_usb_endpoint_descriptor
{
  Uchar  bLength;                                 // Size of this descriptor in bytes
  Uchar  bDescriptorType;                         // ENDPOINT descriptor type
  Uchar  bEndpointAddress;                        // Address of the endpoint
  Uchar  bmAttributes;                            // Endpoint's attributes
  Uint16 wMaxPacketSize;                          // Maximum packet size for this EP
  Uchar  bInterval;                               // Interval for polling EP in ms
} t_usb_endpoint_descriptor;


//--- USB LANGUAGE DESCRIPTOR
typedef struct st_usb_language_descriptor
{
  Uchar  bLength;                                 // size of this descriptor in bytes
  Uchar  bDescriptorType;                         // STRING descriptor type
  Uint16 wlangid;                                 // language id
} t_usb_language_descriptor;


//--- USB MANUFACTURER DESCRIPTOR
typedef struct st_usb_manufacturer
{
  Uchar  bLength;                                 // size of this descriptor in bytes
  Uchar  bDescriptorType;                         // STRING descriptor type
  Uint16 wstring[USB_MN_LENGTH];                  // unicode characters
} t_usb_manufacturer;
     
     
//--- USB PRODUCT DESCRIPTOR
typedef struct st_usb_product
{
  Uchar  bLength;                                 // size of this descriptor in bytes
  Uchar  bDescriptorType;                         // STRING descriptor type
  Uint16 wstring[USB_PN_LENGTH];                  // unicode characters
} t_usb_product;


//--- USB SERIAL NUMBER DESCRIPTOR
typedef struct st_usb_serial_number
{
  Uchar  bLength;                                 // size of this descriptor in bytes
  Uchar  bDescriptorType;                         // STRING descriptor type
  Uint16 wstring[USB_SN_LENGTH];                  // unicode characters
} t_usb_serial_number;





//-------------------------------------------------------------------------------------------------


void 	  usb_init(void);
void    usb_init_pll(void);
#define Usb_stop_pll()                    (PLLCON &= ~MSK_PLLEN)
void    usb_interrupt_process(void) interrupt IRQ_USB;
void    usb_send_data(char sig, char cmd, char *p, unsigned char num);

#endif  // _USB_H_                            

