//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Programm:    - Steuerprogramm zum USB Programmer fr AT89C2051 / AT89C4051
//              - baut eine Verbindung zur USB-Programmer-Schaltung auf und
//                ermglicht das Senden von Befehlen und Empfangen von Status
//                informationen
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include <windows.h>
#include <commctrl.h>
#include <process.h>
#include <stdio.h>
#include <math.h>
#include "usb.h"
#include "log.h"
#include "commands.h"            
#include "resource.h"

#pragma comment (lib,"shlwapi.lib") 

//--- globale Variablen
HINSTANCE         g_hInst;
HWND              g_hMainWnd;
char              g_HexFile[MAX_PATH];
bool              g_isHexFile = FALSE;

static char       g_ProgName[]     = "USB Programmer fr Atmel AT89C2051/4051 Controller";
static char       g_ProgVersion[]  = "1.0";
static char      *g_LockStates[]   =  { "01 - ISP disabled",
                                        "11 - ISP + verify disabled"  
                                      };

#define C_ABOUT_TEXT        "USB Programmer fr Atmel AT89C2051/4051 Controller\n"  \
                            "_______________________________________________\n\n"   \
                            "Laborprojekt SS2005, FH-Wedel\n\n"                     \
                            "Copyright 2005, Andreas Schibilla (ii4900)\n\n"
#define C_ABOUT_CAPTION     "ABOUT:"
#define C_README            "readme.txt"



//*****************************************************************************
// Funktion:    aktiviert oder deaktiviert alle Buttons und Meneintrge
//              im Hauptfenster (keine Aktionen erlaubt, wenn beschftigt...)
//-----------------------------------------------------------------------------
// Parameter:   bEnable - aktivieren=TRUE oder deaktivieren=FALSE
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
void EnableControls(BOOL bEnable)
{
 HMENU hMenu   = GetMenu(g_hMainWnd);
 UINT  uEnable = bEnable ? MF_ENABLED : MF_GRAYED;

 EnableMenuItem(hMenu, IDB_HEXFILE,         uEnable);
 EnableMenuItem(hMenu, IDM_DEBUG,           uEnable);
 EnableMenuItem(hMenu, IDB_TYPE,            uEnable);
 EnableMenuItem(hMenu, IDB_ERASE,           uEnable);
 EnableMenuItem(hMenu, IDB_BLANK_CHECK,     uEnable);
 EnableMenuItem(hMenu, IDB_PROGRAM,         uEnable);
 EnableMenuItem(hMenu, IDB_VERIFY,          uEnable); 
 EnableMenuItem(hMenu, IDB_AUTO,            uEnable);

 EnableWindow (GetDlgItem(g_hMainWnd, IDS_HEXFILE),        bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDE_HEXFILE),        bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_HEXFILE),        bEnable);

 EnableWindow (GetDlgItem(g_hMainWnd, IDG_COMMAND),        bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_ERASE),          bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_BLANK_CHECK),    bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_PROGRAM),        bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_VERIFY),         bEnable);

 EnableWindow (GetDlgItem(g_hMainWnd, IDB_AUTO),           bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_TYPE),           bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDS_LOCKBITS),       bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDC_LOCKBITS),       bEnable);
 EnableWindow (GetDlgItem(g_hMainWnd, IDB_SET_LOCK),       bEnable);

}



//*****************************************************************************
// Funktion:    liest eine Hexdatei ein und prft, ob das Intel-Hex-Format
//              eingehalten wurde.  Ist dies der Fall wird die Anzahl an
//              Datenbytes / Letzte Adresse angezeigt
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle des Hauptfensters
//              filename  - Name der Datei, die geprft werden soll
// Return:      gibt zurck, ob die Datei gltig (TRUE) oder nicht (FALSE) ist
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL CheckHexFile(HWND hwnd, char *filename)
{
 char               line[1025];
 FILE               *stream;
 t_hex_record       hr;
 size_t             num       = 0; 
 unsigned int       last_addr = 0;
 unsigned int       cnt       = 1;

 if( (stream = fopen(filename, "r" )) != NULL )
 {
      while( fgets(line, 1025, stream ) != NULL)              // lese jeweils eine Zeile der Hexdatei ein
      {
            size_t len = strlen(line);                        
            if (!ReadHexRecord(&hr, line, len) )              // extrahiere Hex-Record aus der Zeile
            {
                Log("\r\nERR: Ungltige Hex-Datei! Fehler in Zeile %d\r\n", cnt);
                fclose( stream );
                return FALSE;
            }
            num += hr.len;                                    // Datenbytes aus Hexrecord zhlen
            int offset = (hr.len > 0) ? (hr.len-1) : 0;       // Adressoffset berechnen
            unsigned int uiAddr = hr.addr+offset;             // letze Adresse aus Hexrecord
            if ( uiAddr > last_addr )                         // wenn neue Adresse grer,
                last_addr = uiAddr;                           // dann ist sie "letzte" Adresse

            ++cnt;

      }
      fclose( stream );
 }
 else
 {
    Log("\r\nERR: Konnte Hex-Datei %s nicht zum Lesen ffnen!\r\n", filename);
    return FALSE;
 }
 strcpy(line,"");
 sprintf(line, "%d [%Xh] Bytes", num, num);
 SetDlgItemText(hwnd, IDS_SIZE, line);

 strcpy(line, "");
 sprintf(line, "%d [%Xh]", last_addr, last_addr);
 SetDlgItemText(hwnd, IDS_LAST_ADDRESS, line);

 return TRUE;
}



//*****************************************************************************
// Funktion:    de-/aktiviert die Buttons (nach Auswahl einer Hexdatei)
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle des Hauptfensters
//              b         - aktivieren=TRUE oder deaktivieren=FALSE
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-27, Andreas Schibilla
//*****************************************************************************
void SetControlsOnFileSelect(HWND hwnd, BOOL b)
{
 EnableWindow (GetDlgItem(hwnd, IDB_PROGRAM),  b);
 EnableWindow (GetDlgItem(hwnd, IDB_VERIFY),   b);
 EnableWindow (GetDlgItem(hwnd, IDB_AUTO),     b);
}



//*****************************************************************************
// Funktion:    ffnet einen Dateiauswahldialog und berprft die Gltigkeit
//              der ausgewhlten Datei (HEX / BIN) 
//              Anschlieend werden die Byteinfos angezeigt
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle des Hauptfensters
// Return:      g_HexFile bekommt den Namen der ausgewhlten Datei
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
void LoadHexFile(HWND hwnd)
{
 OPENFILENAME  ofn;
 
 ZeroMemory (&ofn, sizeof(OPENFILENAME));
 ofn.lStructSize      = sizeof(OPENFILENAME);
 ofn.hwndOwner        = hwnd;
 ofn.lpstrFile        = g_HexFile;
 ofn.nMaxFile         = sizeof(g_HexFile);
 ofn.lpstrFilter      = "Intel-Hex-Format (*.hex; *.ihx)\0*.HEX;*.IHX\0Binrdateien (*.bin; *.dat; *.out)\0*.BIN;*.DAT;*.OUT\0Alle Dateien (*.*)\0*.*\0";
 ofn.nFilterIndex     = 1;
 ofn.lpstrFileTitle   = NULL;
 ofn.nMaxFileTitle    = 0;
 ofn.lpstrInitialDir  = NULL;
 ofn.Flags            = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
                    
 // Dateiauswahldialog anzeigen
 if (GetOpenFileName(&ofn) ) 
 {
        SetDlgItemText(hwnd, IDE_HEXFILE,      ""  );
        SetDlgItemText(hwnd, IDS_SIZE,         "?" );
        SetDlgItemText(hwnd, IDS_LAST_ADDRESS, "?" );
        g_isHexFile = FALSE;
        SetControlsOnFileSelect(hwnd, FALSE);
        ClearLog();

        // unterscheide Hex- und Binrdatei an der Erweiterung
        if (isHexFile(g_HexFile) )
        {
            Log("Intel-Hex-Datei ausgewhlt...\r\n");
            if (CheckHexFile(hwnd, g_HexFile))
            {
                SetDlgItemText(hwnd, IDE_HEXFILE, g_HexFile);
                SetControlsOnFileSelect(hwnd, TRUE);
                g_isHexFile = TRUE;
            }

        }
        else
        {                 
            DWORD dwSize;
            if (isBinFile(g_HexFile, &dwSize) )
            {
                char buf[100];
                Log("Binrdatei ausgewhlt...\r\n");
                strcpy(buf,"");
                sprintf(buf, "%d [%Xh] Bytes", dwSize, dwSize);
                SetDlgItemText(hwnd, IDS_SIZE, buf);
                strcpy(buf,"");
                sprintf(buf, "%d [%Xh]", dwSize-1, dwSize-1);
                SetDlgItemText(hwnd, IDS_LAST_ADDRESS, buf);
                SetDlgItemText(hwnd, IDE_HEXFILE, g_HexFile);
                SetControlsOnFileSelect(hwnd, TRUE);
            }
            else
            {
                Log("\r\nERR: Ungltige Datei ausgewhlt!\r\n");                
            }
        } 
 }
}



//*****************************************************************************
// Funktion:    ermittle eigenen Exe-Pfad 
//-----------------------------------------------------------------------------
// Parameter:   str      - Zeiger auf Zielstring
//              len      - max. Gre des Zielstrings
// Return:      gibt einen Zeiger auf den Pfadnamen zurck
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
char * GetExePath(char *str, size_t len)
{ 
 DWORD i = GetModuleFileName(NULL, str, len)-1;    // __argv[0]
 while(str[i] != '\\')
   --i;
 
 str[i+1] = '\0';
 return str;
}



//*****************************************************************************
// Funktion:    Sendet einen Befehl zum Setzen der Lockbits
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle des Hauptfensters
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
void SetLock(HWND hwnd)
{
 // ermitteln, wie die Lockbits gesetzt werden sollen
 // lb=0   ->  "01 - ISP disabled"
 // lb=1   ->  "11 - ISP + verify disabled"
 unsigned char lb = (char) SendDlgItemMessage(hwnd, IDC_LOCKBITS, CB_GETCURSEL, 0, 0);

 // Paket zum Setzen der Lockbits erzeugen und abschicken
 if (chip_set_lockbits(lb+1) )
 {
    Log("Lockbits erfolgreich gesetzt!\r\n");
 }

}



//*****************************************************************************
// Funktion:    Ermittelt die Version des USB Programmers bzw. den
//              eingeleten Chiptyp und zeigt die Infos an
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle des Hauptfensters
// Return:      gibt zurck, ob das Auslesen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL GetChipAndVersion(HWND hwnd)
{
 t_signature  sig;
 
 if (chip_type_and_version(&sig) )
 {
    char buf[100];
    SetDlgItemText(hwnd, IDS_VERSION,      sig.version_str);
    SetDlgItemText(hwnd, IDS_MANUFACTURER, sig.mfr_str);
    SetDlgItemText(hwnd, IDS_PRODUCT,      sig.art_str);
    strcpy(buf, "");
    sprintf(buf, "%d [%Xh] Bytes", sig.flashsize, sig.flashsize);
    SetDlgItemText(hwnd, IDS_FLASHSIZE,   buf);
    return TRUE;
 }
 else
 {
    SetDlgItemText(hwnd, IDS_VERSION,      "?");
    SetDlgItemText(hwnd, IDS_MANUFACTURER, "?");
    SetDlgItemText(hwnd, IDS_PRODUCT,      "?");
    SetDlgItemText(hwnd, IDS_FLASHSIZE,    "?");
    return FALSE;
 }
}



//*****************************************************************************
// Funktion:    Thread-Funktion, die das Programmieren des Chips abwickelt
//-----------------------------------------------------------------------------
// Parameter:   pVoid - Thread-Zeiger, ungenutz
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
VOID ProgramThread(PVOID pvoid)
{
 EnableControls(FALSE);                                

 if (chip_program(g_HexFile, g_isHexFile, CMD_PROGRAM) )
    Log("Das Programmieren des Chips ist abgeschlossen!\r\n");

 EnableControls(TRUE);
}



//*****************************************************************************
// Funktion:    Thread-Funktion, die das Verifzieren des Chips abwickelt
//-----------------------------------------------------------------------------
// Parameter:   pVoid - Thread-Zeiger, ungenutz
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
VOID VerifyThread(PVOID pvoid)
{
 EnableControls(FALSE);                                

 if (chip_program(g_HexFile, g_isHexFile, CMD_VERIFY) )
    Log("Das Verifizieren war erfolgreich!\r\n");

 EnableControls(TRUE);
}



//*****************************************************************************
// Funktion:    Thread-Funktion, die einen kompletten Auto-Programmiervorgang
//              abwickelt
//-----------------------------------------------------------------------------
// Parameter:   pVoid - Thread-Zeiger, ungenutz
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
VOID AutoThread(PVOID pvoid)
{
 DWORD dwTime = GetTickCount();
 
 EnableControls(FALSE);                                
 ClearLog();

 //--- Chip-Typ und Version ermitteln
 if (GetChipAndVersion(g_hMainWnd) )
 {
    Log("Chiptyp und Programmerversion erfolgreich ermittelt!\r\n");
 }
 else
 {
    Log("\r\nERR: Konnte Chiptyp oder Programmerversion nicht ermitteln!\r\n");
    EnableControls(TRUE);
    return;
 }
 Sleep(10);

 //--- Erase
 if (chip_erase() )
    Log("Der Chip wurde erfolgreich gelscht!\r\n");
 else
 {
    Log("\r\nERR: Konnte den Chip nicht lschen!\r\n");
    EnableControls(TRUE);
    return;
 }
 Sleep(10);

 //--- Blank Check
 if (chip_blank_check() )
    Log("Der Blank-Check war erfolgreich!\r\n");
 else
 {
    Log("\r\nERR: Blank-Check war nicht erfolgreich!\r\n");
    EnableControls(TRUE);
    return;
 }
 Sleep(10);

 //--- Program
 if (chip_program(g_HexFile, g_isHexFile, CMD_PROGRAM) )
    Log("Das Programmieren des Chips war erfolgreich!\r\n");
 else
 {
    Log("\r\nERR: Konnte den Chip nicht programmieren!\r\n");
    EnableControls(TRUE);
    return;
 }
 Sleep(10);

 //--- Verify
 if (chip_program(g_HexFile, g_isHexFile, CMD_VERIFY) )
    Log("Das Verifzieren des Chips war erfolgreich!\r\n");
 else
 {
    Log("\r\nERR: Die Verifizierung war nicht erfolgreich!\r\n");
    EnableControls(TRUE);
    return;
 }
 
 dwTime = GetTickCount() - dwTime;
 double dTime = (double) dwTime / 1000.0f;                                               
 Log("Automatisches Programmieren abgeschlossen (%f Sek)!\r\n", dTime);
 EnableControls(TRUE);
}



//*****************************************************************************
// Funktion:    Callback-Routine fr die Bearbeitung aller Nachrichten 
//              des Hauptfensters
//-----------------------------------------------------------------------------
// Parameter:   hwnd      - Fenster-Handle
//              message   - zu behandelnde Nachricht
//              wParam    - optionaler Parameter der Nachricht
//              lParam    - optionaler Parameter der Nachricht
// Return:      Gibt zurck, ob die Nachricht behandelt wurde
//-----------------------------------------------------------------------------
// Datum:       2005-06-05, Andreas Schibilla
//*****************************************************************************
BOOL CALLBACK MainWndProc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
{

 switch (message)
 {

    case WM_INITDIALOG: 
          {                
            // Haupt- und Logfenster initialisieren
            g_hMainWnd = hwnd;
            SetWindowText(hwnd, g_ProgName);
            DWORD dwIndex = SendMessage(GetDlgItem(hwnd, IDC_LOCKBITS), CB_ADDSTRING, 0, (LPARAM) (LPCSTR) g_LockStates[0]); 
            dwIndex = SendMessage(GetDlgItem(hwnd, IDC_LOCKBITS), CB_ADDSTRING, 0, (LPARAM) (LPCSTR) g_LockStates[1]); 
            SendMessage(GetDlgItem(hwnd, IDC_LOCKBITS), CB_SETCURSEL , 0, 0); 
            SetFocus(GetDlgItem(hwnd, IDB_CONNECT));
            HFONT hFontNew = CreateFont( 14, 7, 0, 0, 500, false, false, false, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Courier New");
            SendMessage (GetDlgItem(hwnd, IDE_LOG), WM_SETFONT, (WPARAM) hFontNew, 0) ;
            SetClassLong(hwnd, GCL_HICON, (LONG) LoadIcon(g_hInst, MAKEINTRESOURCE(IDI_MAIN)) ); 
            return FALSE;
          }

   case WM_COMMAND:
          switch (LOWORD(wParam))
          {  
              case IDB_HEXFILE:       LoadHexFile(hwnd);
                                      break;

              case IDB_TYPE:          if ( GetChipAndVersion(hwnd) )
                                          Log("Chiptyp und Programmerversion ermittelt\r\n");
                                      break;

              case IDB_AUTO:          if (GetDlgItemText(hwnd, IDE_HEXFILE, g_HexFile, sizeof(g_HexFile)-1 ) )
                                          _beginthread(AutoThread, 0, NULL);
                                      break;

              case IDB_ERASE:         if (chip_erase() )
                                          Log("Der Chip wurde erfolgreich gelscht!\r\n");
                                      break;

              case IDB_BLANK_CHECK:   if (chip_blank_check() )
                                          Log("Der Blank-Check war erfolgreich!\r\n");
                                      break;

              case IDB_PROGRAM:       if (GetDlgItemText(hwnd, IDE_HEXFILE, g_HexFile, sizeof(g_HexFile)-1 ) )
                                          _beginthread(ProgramThread, 0, NULL);
                                      break;

              case IDB_VERIFY:        if (GetDlgItemText(hwnd, IDE_HEXFILE, g_HexFile, sizeof(g_HexFile)-1 ) )
                                          _beginthread(VerifyThread, 0, NULL);
                                      break;

              case IDB_SET_LOCK:      SetLock(hwnd); 
                                      break;

              case IDM_DEBUG:         chip_debug();
                                      break;

              case IDM_INFO:          {
                                        char buf[MAX_PATH];
                                        GetExePath(buf, sizeof(buf)-1);
                                        strcat(buf, C_README);
                                        ShellExecute(hwnd, "open", buf, NULL, NULL, SW_SHOWMAXIMIZED); 
                                        break;
                                      }

              case IDM_ABOUT:         MessageBox(hwnd, C_ABOUT_TEXT, C_ABOUT_CAPTION, MB_ICONINFORMATION | MB_OK);
                                      break;

              case IDOK:
              case IDCANCEL:          ExitProcess(0);
                                      break;

              default:        break;
          }

    
 }
 return FALSE;
}



//*****************************************************************************
// Funktion:    Diese Eintrittsfunktion erstellt das Hauptfenster und den 
//              Eintritt in die Windows Message Loop.
//-----------------------------------------------------------------------------
// Parameter:   hInstance      - Instance auf die Anwendung 
//              hPrevInstance  - vorige Instance
//              lpCmdLine      - Kommando-Parameter
//              nCmdShow       - Anzeige-Optionen
// Return:      Gibt einen Return-Code an Windows zurck
//-----------------------------------------------------------------------------
// Datum:       2005-05-25, Andreas Schibilla
//*****************************************************************************
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
 MSG         msg;

 g_hInst = hInstance;  

 // Das Hauptfenster erstellen
 g_hMainWnd = CreateDialog(hInstance, "MAINWND", NULL, (DLGPROC) MainWndProc);

 if (g_hMainWnd==NULL)
   MessageBox(NULL, "ERR: Das Hauptfenster konnte nicht erstellt werden!", g_ProgName, MB_ICONERROR);
   
 // Eintritt in Windows Message Loop
 while (GetMessage(&msg, NULL, 0, 0))
 {
      if (g_hMainWnd==0 || !IsDialogMessage(g_hMainWnd, &msg))
      {
          TranslateMessage(&msg);
          DispatchMessage(&msg);
      }
 }
 
 return (msg.wParam);
}

